@@ -1,8 +1,6 @@
 #![forbid(unsafe_code)]
 #![deny(clippy::all)]
 
-use anyhow::Error;
-use rayon::{iter::Either, prelude::*};
 use slog::{debug, error, Logger};
 use sloggers::null::NullLoggerBuilder;
 use sloggers::Build;
@@ -10,6 +8,9 @@ use std::{iter, mem};
 use thiserror::Error;
 use yozuk_sdk::prelude::*;
 
+#[cfg(feature = "rayon")]
+use rayon::prelude::*;
+
 mod labeler;
 mod model;
 mod modelgen;
@@ -54,68 +55,65 @@ impl Yozuk {
 
         let labeler = FeatureLabeler::new(&self.labelers);
 
-        let pool = rayon::ThreadPoolBuilder::new()
-            .num_threads(num_cpus::get().max(2))
-            .build()
-            .unwrap();
+        #[cfg(feature = "rayon")]
+        let iter = self.commands.par_iter();
+        #[cfg(not(feature = "rayon"))]
+        let iter = self.commands.iter();
 
-        pool.install(|| {
-            let mut commands = self
-                .commands
-                .par_iter()
-                .filter_map(|cache| cache.as_ref())
-                .map(|cache| {
-                    (
-                        cache,
-                        cache
-                            .preprocessors
-                            .iter()
-                            .fold(tokens.to_vec(), |tokens, prep| prep.preprocess(tokens)),
-                    )
-                })
-                .map(|(cache, tokens)| {
-                    (
-                        cache,
-                        if let Some(model) = &cache.model {
-                            model.tag_tokens(&labeler, &tokens)
-                        } else {
-                            tokens
-                        },
-                    )
-                })
-                .filter_map(|(cache, args)| {
+        let mut commands = iter
+            .filter_map(|cache| cache.as_ref())
+            .map(|cache| {
+                (
+                    cache,
                     cache
-                        .translators
-                        .par_iter()
-                        .find_map_first(|tr| tr.parse(&args, streams))
-                        .map(|args| {
-                            (
-                                cache.command.priority(),
-                                CommandArgs {
-                                    args: iter::once(cache.name.to_string())
-                                        .chain(args.args.into_iter())
-                                        .collect(),
-                                    data: args.data,
-                                },
-                            )
-                        })
-                })
-                .collect::<Vec<_>>();
+                        .preprocessors
+                        .iter()
+                        .fold(tokens.to_vec(), |tokens, prep| prep.preprocess(tokens)),
+                )
+            })
+            .map(|(cache, tokens)| {
+                (
+                    cache,
+                    if let Some(model) = &cache.model {
+                        model.tag_tokens(&labeler, &tokens)
+                    } else {
+                        tokens
+                    },
+                )
+            })
+            .filter_map(|(cache, args)| {
+                cache
+                    .translators
+                    .iter()
+                    .find_map(|tr| tr.parse(&args, streams))
+                    .map(|args| {
+                        (
+                            cache.command.priority(),
+                            CommandArgs {
+                                args: iter::once(cache.name.to_string())
+                                    .chain(args.args.into_iter())
+                                    .collect(),
+                                data: args.data,
+                            },
+                        )
+                    })
+            })
+            .collect::<Vec<_>>();
 
-            if commands.is_empty() {
-                let suggest = self.suggest(tokens);
-                return Err(YozukError::UnintelligibleRequest { suggest });
-            }
+        if commands.is_empty() {
+            let suggest = self.suggest(tokens);
+            return Err(YozukError::UnintelligibleRequest { suggest });
+        }
 
-            commands.sort_by_key(|command| -command.0);
-            Ok(commands.into_iter().map(|command| command.1).collect())
-        })
+        commands.sort_by_key(|command| -command.0);
+        Ok(commands.into_iter().map(|command| command.1).collect())
     }
 
     pub fn run_commands(
         &self,
         commands: Vec<CommandArgs>,
         streams: &mut [InputStream],
+        locale: &Locale,
     ) -> Result<Output, YozukError> {
         let commands = commands.into_iter().filter_map(|args| {
             self.model
@@ -127,7 +125,7 @@ impl Yozuk {
         let mut errors = Vec::new();
         for (args, command) in commands {
             let name = args.args[0].clone();
-            match command.run(args, streams) {
+            match command.run(args, streams, locale) {
                 Ok(result) => return Ok(result),
                 Err(err) => errors.push(err.into_output(name)),
             }
@@ -144,7 +142,7 @@ impl Yozuk {
         let string = words.join(" ");
 
         self.skills
-            .par_iter()
+            .iter()
             .flat_map(|cache| &cache.skill.suggests)
             .flat_map(|suggests| suggests.suggests(tokens))
             .map(|s| {
@@ -182,8 +180,12 @@ impl YozukBuilder {
             .logger(self.logger.clone())
             .build_info(build_info);
 
-        let (mut skills, errors): (Vec<_>, Vec<(&'static NamedSkillEntry, Error)>) = skill::SKILLS
-            .par_iter()
+        #[cfg(feature = "rayon")]
+        let iter = skill::SKILLS.par_iter();
+        #[cfg(not(feature = "rayon"))]
+        let iter = skill::SKILLS.iter();
+
+        let results = iter
             .map(|entry| {
                 let config = self
                     .config
@@ -198,15 +200,19 @@ impl YozukBuilder {
                     skill: (entry.entry.init)(&env, &config).map_err(|err| (entry, err))?,
                 })
             })
-            .partition_map(|result| match result {
-                Ok(skill) => Either::Left(skill),
-                Err(err) => Either::Right(err),
-            });
+            .collect::<Vec<_>>();
 
-        for (entry, err) in errors {
-            error!(self.logger, "Failed to initialize {}: {}", entry.key, err);
+        for result in &results {
+            if let Err((entry, err)) = result {
+                error!(self.logger, "Failed to initialize {}: {}", entry.key, err);
+            }
         }
 
+        let mut skills = results
+            .into_iter()
+            .filter_map(|result| result.ok())
+            .collect::<Vec<_>>();
+
         let labelers = skills
             .iter_mut()
             .flat_map(|cache| std::mem::take(&mut cache.skill.labelers))
